package javax.xml.ws.spi;

import java.util.Map;
import javax.xml.ws.wsaddressing.W3CEndpointReference;
import org.w3c.dom.Element;
import java.util.List;
import javax.xml.ws.EndpointReference;
import javax.xml.transform.Source;
import javax.xml.ws.Endpoint;
import javax.xml.ws.WebServiceFeature;
import javax.xml.ws.Service;
import javax.xml.namespace.QName;
import java.util.Iterator;
import java.net.URL;
import javax.xml.ws.WebServiceException;
import java.lang.reflect.Method;

public abstract class Provider
{
    public static final String JAXWSPROVIDER_PROPERTY = "javax.xml.ws.spi.Provider";
    static final String DEFAULT_JAXWSPROVIDER = "com.sun.xml.internal.ws.spi.ProviderImpl";
    private static final Method loadMethod;
    private static final Method iteratorMethod;
    
    protected Provider() {
    }
    
    public static Provider provider() {
        try {
            Object provider = getProviderUsingServiceLoader();
            if (provider == null) {
                provider = FactoryFinder.find("javax.xml.ws.spi.Provider", "com.sun.xml.internal.ws.spi.ProviderImpl");
            }
            if (!(provider instanceof Provider)) {
                Class<Object> pClass = Provider.class;
                String classnameAsResource = pClass.getName().replace('.', '/') + ".class";
                ClassLoader loader = pClass.getClassLoader();
                if (loader == null) {
                    loader = ClassLoader.getSystemClassLoader();
                }
                URL targetTypeURL = loader.getResource(classnameAsResource);
                throw new LinkageError("ClassCastException: attempting to cast" + provider.getClass().getClassLoader().getResource(classnameAsResource) + "to" + targetTypeURL.toString());
            }
            else {
                return (Provider)provider;
            }
        }
        catch (WebServiceException ex) {
            throw ex;
        }
        catch (Exception ex2) {
            throw new WebServiceException("Unable to createEndpointReference Provider", ex2);
        }
    }
    
    private static Provider getProviderUsingServiceLoader() {
        if (Provider.loadMethod != null) {
            Object loader;
            try {
                loader = Provider.loadMethod.invoke(null, new Object[] { Provider.class });
            }
            catch (Exception e) {
                throw new WebServiceException("Cannot invoke java.util.ServiceLoader#load()", e);
            }
            Iterator<Provider> it;
            try {
                it = (Iterator)Provider.iteratorMethod.invoke(loader, new Object[0]);
            }
            catch (Exception e2) {
                throw new WebServiceException("Cannot invoke java.util.ServiceLoader#iterator()", e2);
            }
            return it.hasNext() ? ((Provider)it.next()) : null;
        }
        return null;
    }
    
    public abstract ServiceDelegate createServiceDelegate(URL p0, QName p1, Class<? extends Service> p2);
    
    public ServiceDelegate createServiceDelegate(URL wsdlDocumentLocation, QName serviceName, Class<? extends Service> serviceClass, WebServiceFeature... features) {
        throw new UnsupportedOperationException("JAX-WS 2.2 implementation must override this default behaviour.");
    }
    
    public abstract Endpoint createEndpoint(String p0, Object p1);
    
    public abstract Endpoint createAndPublishEndpoint(String p0, Object p1);
    
    public abstract EndpointReference readEndpointReference(Source p0);
    
    public abstract <T> T getPort(EndpointReference p0, Class<T> p1, WebServiceFeature... p2);
    
    public abstract W3CEndpointReference createW3CEndpointReference(String p0, QName p1, QName p2, List<Element> p3, String p4, List<Element> p5);
    
    public W3CEndpointReference createW3CEndpointReference(String address, QName interfaceName, QName serviceName, QName portName, List<Element> metadata, String wsdlDocumentLocation, List<Element> referenceParameters, List<Element> elements, Map<QName, String> attributes) {
        throw new UnsupportedOperationException("JAX-WS 2.2 implementation must override this default behaviour.");
    }
    
    public Endpoint createAndPublishEndpoint(String address, Object implementor, WebServiceFeature... features) {
        throw new UnsupportedOperationException("JAX-WS 2.2 implementation must override this default behaviour.");
    }
    
    public Endpoint createEndpoint(String bindingId, Object implementor, WebServiceFeature... features) {
        throw new UnsupportedOperationException("JAX-WS 2.2 implementation must override this default behaviour.");
    }
    
    public Endpoint createEndpoint(String bindingId, Class<?> implementorClass, Invoker invoker, WebServiceFeature... features) {
        throw new UnsupportedOperationException("JAX-WS 2.2 implementation must override this default behaviour.");
    }
    
    static {
        Method tLoadMethod = null;
        Method tIteratorMethod = null;
        try {
            Class<?> clazz = Class.forName("java.util.ServiceLoader");
            tLoadMethod = clazz.getMethod("load", new Class[] { Class.class });
            tIteratorMethod = clazz.getMethod("iterator", new Class[0]);
        }
        catch (ClassNotFoundException ex) {}
        catch (NoSuchMethodException ex2) {}
        loadMethod = tLoadMethod;
        iteratorMethod = tIteratorMethod;
    }
}
